import javax.jms.*;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.util.Hashtable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.logging.Level;

/**
 * User: rnell
 * Date: 10/29/12
 * Time: 10:53 AM
 */
public class QueueReceive2 {
    //public final static String JNDI_FACTORY="org.jnp.interfaces.NamingContextFactory";
    public final static String JNDI_FACTORY = "org.jboss.naming.remote.client.InitialContextFactory";
    //*************** Connection Factory JNDI name *************************
    public final static String JMS_FACTORY = "/jms/RemoteConnectionFactory";
//*************** Queue JNDI name *************************

    public final static String QUEUE = "TestQ2";
    private QueueConnectionFactory qconFactory;
    private QueueConnection qcon;
    private QueueSession qsession;
    private QueueReceiver qreceiver;
    private Queue queue;
    private boolean quit = false;

    private void processMessage() {
        boolean quit = false;
        while( !quit ) {
            try {
                Message msg = qreceiver.receive();
                quit = onMessage( msg );

            }
            catch( JMSException e ) {
                System.err.println( "Receive failed: " + e );
            }
        }
    }

    public boolean onMessage( Message msg ) {
        boolean quit = false;
        try {
            String msgText;
            if( msg instanceof TextMessage ) {
                msgText = ( ( TextMessage ) msg ).getText();
            } else {
                msgText = msg.toString();
            }
            System.out.println( "<Msg_Receiver>; " + msgText );

            //RN
            if( msgText.equalsIgnoreCase( "rollback")) {
                qsession.rollback();
                System.out.println( "rollback" );
            }
            else {
                qsession.commit();
                System.out.println( "commit" );
            }

            quit = msgText.equalsIgnoreCase( "quit" );
        }
        catch( JMSException jmse ) {
            jmse.printStackTrace();
        }
        return quit;
    }

    public void init( Context ctx, String queueName ) throws NamingException, JMSException {
        qconFactory = ( QueueConnectionFactory ) ctx.lookup( JMS_FACTORY );
        qcon = qconFactory.createQueueConnection( "admin1", "admin123" ); // Provide the application user name and  password with guest role

        //RN
        qsession = qcon.createQueueSession( true, 0 );

        Object c = ctx.lookup( queueName );
        System.out.println( c.toString() );

        queue = ( Queue ) ctx.lookup( queueName );
        qreceiver = qsession.createReceiver( queue );
        qcon.start();
    }

    public void close() throws JMSException {
        qreceiver.close();
        qsession.close();
        qcon.close();
    }

    public static void main( final String[] args ) throws Exception {
        if( args.length != 1 ) {
            System.out.println( "Usage: java QueueReceive URL" );
            return;
        }

        InitialContext ic = getInitialContext( args[ 0 ] );
        QueueReceive2 qr = new QueueReceive2();

        System.out.println( "JMS Ready To Receive Messages (To quit, send a  message from QueueSender.class)." );

        //Wait until a "quit" message has been received.

        qr.init( ic, QUEUE );
        qr.processMessage();
        qr.close();
    }

    private static InitialContext getInitialContext( String url ) throws NamingException {
        Hashtable env = new Hashtable();
        env.put( Context.INITIAL_CONTEXT_FACTORY, JNDI_FACTORY );
        env.put( Context.PROVIDER_URL, url );
        return new InitialContext( env );
    }

}
